from bitcash import PrivateKeyTestnet, format

import gcl_secrets

initiator_key = PrivateKeyTestnet(gcl_secrets.INITIATOR_PRIVATE_KEY)
print(initiator_key.public_key)
print(len(initiator_key.public_key))

message = b"123fsafsafsfsfsafsaffsafse3223gsdgbdbJGoeffffddw23abc"

sig = initiator_key.sign(message)
print(sig)
print(len(sig))
pubkey = initiator_key.public_key
print(len(initiator_key.public_key))

verified = format.verify_sig(sig, message, pubkey)

print(verified)





from bitcash import PrivateKeyTestnet

import gcl_secrets, time


initiator_key = PrivateKeyTestnet(gcl_secrets.INITIATOR_PRIVATE_KEY)

print(initiator_key.get_unspents())

print(initiator_key.get_balance())

to_send = [('bchtest:qqtp5le4dp59tmavqdsapc5u0n2qm89rfuty833q5u', 1112, 'satoshi')]


next_to_spend_txid = None
# next_to_spend_txid = "7ff29e72dfbce5a631699175d6ef6b16c9acbcfc0a424b765378602de9d1bb59s"


if not next_to_spend_txid:
    next_to_spend_txid = initiator_key.get_unspents()[0].txid

next_tx_found = False

for i in range(30):
    while not next_tx_found:
        unspents = initiator_key.get_unspents()
        for unspent in unspents:
            if unspent.txid == next_to_spend_txid:
                to_spend = [unspent]
                next_tx_found = True
                print(unspent)
        if not next_tx_found:
            time.sleep(1)

    next_to_spend_txid = initiator_key.send(to_send, unspents=to_spend)
    print(next_to_spend_txid)
    next_tx_found = False

    print(i)





import os
import json
import gcl_message, gamechain_monitor, message_monitors, gcl_message_builder, gcl_parser
import utils

NEXT_TO_SPEND_TXID = "next_txid"


class GameChainLobbyClient:

    def __init__(self, private_key, save_file):
        self._private_key = private_key
        self._save_file = save_file

        if not os.path.exists(save_file):
            init_data = self._create_init_data()
            self._data = init_data
            self._save_state()

        self._load_state()

        self._gamechain_monitor = gamechain_monitor.GameChainMonitor(60)

    def _create_init_data(self):
        unspents = self._private_key.get_unspents()
        unspents.sort(key=lambda u: u.amount, reverse=True)
        next_to_use_tx = unspents[0]
        init_data = {
            NEXT_TO_SPEND_TXID: next_to_use_tx.txid
        }
        return init_data

    def _load_state(self):
        with open(self._save_file) as json_file:
            self._data = json.load(json_file)

    def _save_state(self):
        print("Saving state: %s" % self._data)
        with open(self._save_file, 'w') as outfile:
            json.dump(self._data, outfile)

    def look_for_game(self, gameshelf_id, label, initial_conditions):
        setup_info = f"{initial_conditions},{label}"
        lfg_message = gcl_message_builder.create_looking_for_game_message(self._private_key, setup_info)
        tx_id = gcl_message.send_message(self._private_key, self._data[NEXT_TO_SPEND_TXID],
                                         gameshelf_id, lfg_message)

        self._data[NEXT_TO_SPEND_TXID] = tx_id
        self._save_state()

        return tx_id

    def wait_for_lfg(self, gameshelf_id):
        cm = message_monitors.LfgMonitor(self._gamechain_monitor, gameshelf_id)
        challenge_gcl_msg = cm.wait_for_challenge()
        return challenge_gcl_msg

    def offer_willing_to_play(self, lfg_addr, lfg_txid_bytes, alternative_conditions):
        wtp_message = gcl_message_builder.create_willing_to_play_message(self._private_key, lfg_txid_bytes, alternative_conditions)

        p_lfg_addr = utils.ensure_prefixed_address(lfg_addr)

        tx_id = gcl_message.send_message(self._private_key, self._data[NEXT_TO_SPEND_TXID],
                                         p_lfg_addr, wtp_message)

        self._data[NEXT_TO_SPEND_TXID] = tx_id
        self._save_state()

        return tx_id

    def wait_for_challenge_response(self, wtp_tx_id, public_key_bytes):
        cm = message_monitors.ChallengeResponseMonitor(self._gamechain_monitor, self._private_key.address,
                                                       wtp_tx_id, public_key_bytes)
        cr_gcl_msg = cm.wait_for_challenge_reponse()
        return cr_gcl_msg

    def stop(self):
        self._gamechain_monitor.stop()

    def wait_for_wtp(self, lfg_txid):
        my_addr = self._private_key.address
        mon = message_monitors.WtpMonitor(self._gamechain_monitor, my_addr, lfg_txid)
        wtp_gcl_msg = mon.wait_for_wtp()
        return wtp_gcl_msg

    def accept_wtp(self, sender_addr, wtp_txid_bytes, init_game_msg):
        accept_wtp_message_bytes = gcl_message_builder.create_accept_wtp_message(self._private_key, wtp_txid_bytes,
                                                                                 init_game_msg)

        p_sender_addr = utils.ensure_prefixed_address(sender_addr)

        tx_id = gcl_message.send_message(self._private_key, self._data[NEXT_TO_SPEND_TXID],
                                         p_sender_addr, accept_wtp_message_bytes)

        self._data[NEXT_TO_SPEND_TXID] = tx_id
        self._save_state()

        return tx_id

    def reject_wtp(self, sender_addr, reject_msg):
        # reject_wtp_message = self._create_reject_wtp(self._private_key.address)
        reject_wtp_msg_bytes = reject_msg.encode()
        p_sender_addr = utils.ensure_prefixed_address(sender_addr)

        tx_id = gcl_message.send_message(self._private_key, self._data[NEXT_TO_SPEND_TXID],
                                         p_sender_addr, reject_wtp_msg_bytes)

        self._data[NEXT_TO_SPEND_TXID] = tx_id
        self._save_state()

        return tx_id

    def send_gc_message(self, receiver_addr, msg_body):
        p_receiver_addr = utils.ensure_prefixed_address(receiver_addr)

        tx_id = gcl_message.send_message(self._private_key, self._data[NEXT_TO_SPEND_TXID],
                                         p_receiver_addr, msg_body)

        self._data[NEXT_TO_SPEND_TXID] = tx_id
        self._save_state()

        return tx_id



import asyncio
import cashaddress
import game_shelves, gcl_message

MONITOR_INTERVAL = 0.2


class MessageMonitor:

    def __init__(self, gc_monitor):
        self._monitor = gc_monitor
        self._keep_monitoring = True
        self._response_txid = None
        self._response_gcl_msg = None

    async def _await_response(self):
        while self._keep_monitoring:
            if self._response_txid is not None:
                response_txid = self._response_txid
                self._response_txid = None

                response_gcl_msg = self._response_gcl_msg
                if self._response_gcl_msg is None:
                    response_gcl_msg = gcl_message.receive_message_by_txid(response_txid)

                self._response_gcl_msg = None

                return response_gcl_msg

            await asyncio.sleep(MONITOR_INTERVAL)

    def await_response(self, message_filter):
        self._monitor.register_tx_handler(message_filter)

        loop = asyncio.get_event_loop()
        response_gcl_msg = loop.run_until_complete(self._await_response())

        self._monitor.un_register_tx_handler(message_filter)
        return response_gcl_msg

    def set_response_txid(self, response_txid):
        self._response_txid = response_txid

    def set_response_tx(self, response_txid, gcl_msg):
        self._response_txid = response_txid
        self._response_gcl_msg = gcl_msg


class LfgMonitor(MessageMonitor):

    def __init__(self, gc_monitor, game_id):
        super().__init__(gc_monitor)
        self._game_id = game_id

    def _filter_tx_for_challenge(self, tx):
        if gcl_message.check_if_message_is_for_addr(tx, self._game_id):
            txid = tx['txid']
            # self.set_response(txid)
            # print(f"Challenged offered: {txid}")

            # msg_bytes, sender_addr, receiver_addr = gc_message.receive_message_by_txid(txid)
            # if msg_bytes[0] == gc_message.MSG_LFG:
            #     self.set_response(txid)
            #     print(f"Challenged offered: {txid}")
            received_msg = gcl_message.receive_message_by_txid(txid)
            if received_msg.msg_type == gcl_message.MSG_LFG:
                self.set_response_txid(txid)
                print(f"LFG Found: {txid}")
            # return GclMessage(txid, sender_addr, receiver_addr, msg_type, msg_contents)


    def wait_for_challenge(self):
        print("AWAITING RESPONSE")
        return self.await_response(self._filter_tx_for_challenge)


class WtpMonitor(MessageMonitor):

    def __init__(self, gc_monitor, listener_addr, lfg_txid):
        super().__init__(gc_monitor)

        self._listener_addr = listener_addr
        self._lfg_txid = lfg_txid

    def _filter_tx_for_wtp(self, tx):
        if gcl_message.check_if_message_is_for_addr(tx, self._listener_addr):
            txid = tx["txid"]
            wtp_gcl_msg = gcl_message.receive_message_by_txid(txid)
            if wtp_gcl_msg.msg_type == gcl_message.MSG_WTP:
                # check if self._lfg_tx_id matches expected LFG txid
                self.set_response_tx(txid, wtp_gcl_msg)
                print(f"WTP offered: {tx}")

    def wait_for_wtp(self):
        print("AWAITING WTP")
        return self.await_response(self._filter_tx_for_wtp)


class ChallengeResponseMonitor(MessageMonitor):

    def __init__(self, gc_monitor, listener_addr, wtp_tx_id, sender_public_key):
        super().__init__(gc_monitor)

        self._listener_addr = listener_addr
        self._wtp_tx_id = wtp_tx_id
        self._sender_public_key = sender_public_key

    def _filter_tx_for_challenge_response(self, tx):
        if gcl_message.check_if_message_is_for_addr(tx, self._listener_addr):
            txid = tx['txid']
            print(f"Challenge responded: {txid}")

            challenge_gcl_msg = gcl_message.receive_message_by_txid(txid, self._sender_public_key)
            if challenge_gcl_msg.msg_type == gcl_message.MSG_ACC or challenge_gcl_msg.msg.msg_type == gcl_message.MSG_CAN:
                # check if self._wtp_tx_id matches expected WPT txid
                self.set_response_tx(txid, challenge_gcl_msg)
            # if msg_bytes[0] == gc_message.MSG_LFG:
            #     self.set_response(txid)
            #     print(f"Challenged offered: {txid}")

    def wait_for_challenge_reponse(self):
        print("AWAITING CHALLENGE RESPONSE")
        return self.await_response(self._filter_tx_for_challenge_response)


# class ChallengeMonitor:
#
#     def __init__(self, gc_monitor, game_id):
#         self._monitor = gc_monitor
#         self._game_id = game_id
#         self._challenge_tx = None
#         self._keep_monitoring = True
#
#     def _filter_tx_for_challenge(self, tx):
#         print(f"Got it: {tx}")
#         legacy_dest_addr = next(iter(tx['vout'][1].keys()))
#         dest_addr = cashaddress.convert.to_cash_address(legacy_dest_addr)
#
#         if dest_addr == game_shelves.GAMESHELF_ID_TIC_TAC_TOE:
#             self._challenge_tx = tx
#             print(f"Challenged offered: {tx['txid']}")
#
#     async def _monitor_for_challenge(self):
#         while self._keep_monitoring:
#             if self._challenge_tx is not None:
#                 challenge_tx = self._challenge_tx
#                 self._challenge_tx = None
#                 return challenge_tx
#
#             await asyncio.sleep(MONITOR_INTERVAL)
#
#     def wait_for_challenge(self):
#         self._monitor.register_tx_handler(self._filter_tx_for_challenge)
#         loop = asyncio.get_event_loop()
#         challenge_tx = loop.run_until_complete(self._monitor_for_challenge())
#         return challenge_tx


#
# import asyncio
# import time
#
#
# class InitiatorMonitor:
#     async def wait_for_lfg(self):
#         print("PRE")
#         time.sleep(1)
#         print("POST")
#         yield 123
#
#
# async def main():
#     im = InitiatorMonitor()
#     v = await im.wait_for_lfg()
#
#     print(v)
#
#
# if __name__ == "__main__":
#     loop = asyncio.get_event_loop()
#     loop.run_until_complete(main())



import asyncio
import datetime

class InitiatorMonitor:
    async def display_date(self, loop):
        end_time = loop.time() + 5.0
        while True:
            print(datetime.datetime.now())
            if (loop.time() + 1.0) >= end_time:
                return 123
                break
            await asyncio.sleep(1)

im = InitiatorMonitor()

loop = asyncio.get_event_loop()
# Blocking call which returns when the display_date() coroutine is done
x = loop.run_until_complete(im.display_date(loop))
print(x)
loop.close()



import time
from bitcash import transaction, network
import cashaddress
import utils
import gcl_parser, gc_comm

GCL_PREFIX = 1337
GC_PREFIX = 31337

MSG_LFG = 0x01
MSG_WTP = 0x02
MSG_ACC = 0x03
MSG_REJ = 0x04
MSG_CAN = 0x05

FEE_AMOUNT = 250
MESSAGE_AMOUNT = 0
AMOUNT_UNITS ='satoshi'
MESSAGE_LIMIT = 220  # 100

OP_PUSHDATA = 0x4C


class GclMessage:
    def __init__(self, txid, sender_addr, receiver_addr, msg_type, msg):
        self.txid = txid
        self.sender_addr = utils.ensure_prefixed_address(sender_addr)
        self.receiver_addr = utils.ensure_prefixed_address(receiver_addr)
        self.msg_type = msg_type
        self.msg = msg

    @property
    def txid_bytes(self):
        return bytes.fromhex(self.txid)


class MsgLfg:
    def __init__(self, public_key_bytes, msg_data):
        self.public_key_bytes = public_key_bytes
        self.msg_data = msg_data


class MsgWtp:
    def __init__(self, public_key_bytes, lfg_txid_bytes, msg_data):
        self.public_key_bytes = public_key_bytes
        self.lfg_txid_bytes = lfg_txid_bytes
        self.msg_data = msg_data


class MsgAcc:
    def __init__(self, wtp_txid_bytes, msg_data):
        self.wtp_txid_bytes = wtp_txid_bytes
        self.msg_data = msg_data


class MsgCan:
    def __init__(self, wtp_txid_bytes, msg_data):
        self.wtp_txid_bytes = wtp_txid_bytes
        self.msg_data = msg_data


class MsgRej:
    def __init__(self, msg_data):
        self.msg_data = msg_data


def get_fee_amount(op_return_data, msg_count=0):
    op_return_fee_multiplier = 1
    if msg_count > 1:
        op_return_fee_multiplier = 3

    return FEE_AMOUNT + (op_return_fee_multiplier * len(op_return_data))


def get_unspent_by_txid(sender_key, next_to_spend_txid):
    next_tx_found = False
    while not next_tx_found:
        unspents = sender_key.get_unspents()
        # unspents.sort(key=lambda x: x.amount, reverse=True)
        for unspent in unspents:
            if unspent.txid == next_to_spend_txid:
                return unspent

        if not next_tx_found:
            time.sleep(1)


def send_message(sender_key, spend_from_txid, receiver_addr, op_ret_data):
    # if len(op_ret_data) > MESSAGE_LIMIT:
    #     print(op_ret_data)
    #     raise Exception("Message is too long")

    unspent = get_unspent_by_txid(sender_key, spend_from_txid)
    print(unspent)

    def chunk_data(data, size):
        return (data[i:i + size] for i in range(0, len(data), size))

    messages = []

    if op_ret_data:
        # message_chunks = chunk_data(op_ret_data.encode('utf-8'), MESSAGE_LIMIT)
        message_chunks = chunk_data(op_ret_data, MESSAGE_LIMIT)

        for message in message_chunks:
            messages.append((message, None))

    fee_amount = get_fee_amount(op_ret_data, len(messages))
    amount_back_to_sender = unspent.amount - fee_amount - MESSAGE_AMOUNT
    outputs = [
        (sender_key.address, amount_back_to_sender),
        (receiver_addr, MESSAGE_AMOUNT),
    ]

    outputs.extend(messages)

    tx_hex = transaction.create_p2pkh_transaction(sender_key, [unspent], outputs)
    tx_id = transaction.calc_txid(tx_hex)

    network.NetworkAPI.broadcast_tx_testnet(tx_hex)

    return tx_id


def _parse_message_from_op_return_msg(op_return_msg):
    msg_bytes = bytes.fromhex(op_return_msg)

    return msg_bytes


# #OP_RETURN_PREFIX = "OP_RETURN "
# OP_RETURN_PREFIX = "6a"
# # OP_RETURN_AND_SIZE_BYTES_COUNT = 4
# def receive_message_by_txid(txid, sender_public_key=None) -> GclMessage:
#     tx = utils.get_tx_testnet(txid)
#     vouts = tx["vout"]
#     addr_vouts = [vout for vout in vouts if "addresses" in vout["scriptPubKey"].keys()]
#     sender_addr = addr_vouts[0]["scriptPubKey"]["addresses"][0]
#     receiver_addr = addr_vouts[1]["scriptPubKey"]["addresses"][0]
#     # op_return_msg = next(vout["scriptPubKey"]["hex"][OP_RETURN_AND_SIZE_BYTES_COUNT:] for vout in vouts if vout["scriptPubKey"]["asm"].startswith(OP_RETURN_PREFIX))
#     # op_return_asm = next(vout["scriptPubKey"]["asm"][2:] for vout in vouts if
#     #                      vout["scriptPubKey"]["asm"].startswith(OP_RETURN_PREFIX))
#     op_return_asms = [vout["scriptPubKey"]["hex"][2:] for vout in vouts if
#                       vout["scriptPubKey"]["hex"].startswith(OP_RETURN_PREFIX)]
#     op_return_asm = op_return_asms[0]
#     msg_bytes = bytes.fromhex(op_return_asm)
#
#     # msg_bytes = _parse_message_from_op_return_msg(op_return_msg)
#     print(f"TX_ID: {txid}")
#     print(f"MSG_ASM: {op_return_asm}")
#     msg_type, msg_contents = gcl_parser.parse_op_return(msg_bytes, sender_public_key)
#
#     return GclMessage(txid, sender_addr, receiver_addr, msg_type, msg_contents)

# TURN_PREFIX = "OP_RETURN "
# OP_RETURN_PREFIX = "6a"
# OP_RETURN_AND_SIZE_BYTES_COUNT = 4
def receive_message_by_txid(txid, sender_public_key=None) -> GclMessage:
    sender_addr, receiver_addr, op_return_asms = gc_comm.get_sender_receiver_op_returns_by_txid(txid)
    op_return_asm_bytes = op_return_asms[0]

    print(f"TX_ID: {txid}")
    print(f"MSG_ASM: {op_return_asm_bytes}")
    msg_type, msg_contents = gcl_parser.parse_op_return(op_return_asm_bytes, sender_public_key)

    return GclMessage(txid, sender_addr, receiver_addr, msg_type, msg_contents)


def check_if_message_is_for_addr(tx, addr):
    vouts = tx["vout"]
    if len(vouts) < 2:
        return False

    receiver_addr = next(iter(vouts[1].keys()))
    receiver_addr = cashaddress.convert.to_cash_address(receiver_addr)
    receiver_addr = utils.ensure_prefixed_address(receiver_addr)

    return receiver_addr == addr


if __name__ == "__main__":
    print("MSG")
    msg, sender, receiver = receive_message_by_txid("a90285a9270c980b860f6f0921ec887395778a400e1e3c850dc0fac8dabba23d")
    # h = int("0x" + msg, 16)
    print(msg)
    # h = bytes.fromhex(msg)
    # print(h)




#
# const transaction = new bch.Transaction()
#         .from(utxo)
#         .to(toAddr, 0)
#         .change(fromAddr)
#         .fee(FEE)
#         .addData(message)
#         .sign(fromPk);



import gcl_message


def _get_prefix_bytes():
    return 0x0400001337.to_bytes(5, 'big')


def _get_op_pushdata_bytes(message_body_len):
    return gcl_message.OP_PUSHDATA.to_bytes(1, 'big') + message_body_len.to_bytes(1, 'big')


def _get_version_bytes():
    return 0x01.to_bytes(1, 'big')


def create_looking_for_game_message(sender_key, setup_info):
    lfg_msg_bytes = gcl_message.MSG_LFG.to_bytes(1, 'big')
    if len(sender_key.public_key) != 33:
        raise Exception("public_key_bytes must be 33 bytes long")

    msg_bytes = setup_info.encode()
    signed_msg_bytes = sender_key.sign(msg_bytes)

    msg_size = len(msg_bytes).to_bytes(1, 'big')
    signed_msg_size = len(signed_msg_bytes).to_bytes(1, 'big')

    message_body = [
        lfg_msg_bytes,
        sender_key.public_key,
        msg_size,
        signed_msg_size,
        msg_bytes,
        signed_msg_bytes
    ]
    message_body_bytes = b''.join(message_body)
    message_body_len = len(message_body_bytes)

    message_parts = [
        _get_prefix_bytes(),
        _get_op_pushdata_bytes(message_body_len),
        _get_version_bytes(),
        message_body_bytes
    ]

    message = b''.join(message_parts)
    print("MSG: %s" % message)
    return message


def create_willing_to_play_message(sender_key, lfg_txid_bytes, setup_response):
    wtp_msg_bytes = gcl_message.MSG_WTP.to_bytes(1, 'big')
    if len(lfg_txid_bytes) != 32:
        raise Exception("lfg_txid must be 32 bytes long")
    if len(sender_key.public_key) != 33:
        raise Exception("public_key_bytes must be 33 bytes long")

    setup_response_bytes = setup_response.encode()
    signed_lfg_txid = sender_key.sign(lfg_txid_bytes)

    ac_size = len(setup_response_bytes).to_bytes(1, 'big')
    signed_txid_size = len(signed_lfg_txid).to_bytes(1, 'big')

    message_body = [
        wtp_msg_bytes,
        lfg_txid_bytes,
        sender_key.public_key,
        ac_size,
        signed_txid_size,
        setup_response_bytes,
        signed_lfg_txid
    ]
    message_body_bytes = b''.join(message_body)
    message_body_len = len(message_body_bytes)

    message_parts = [
        _get_prefix_bytes(),
        _get_op_pushdata_bytes(message_body_len),
        _get_version_bytes(),
        message_body_bytes
    ]

    message = b''.join(message_parts)
    return message


def create_accept_wtp_message(sender_key, wtp_txid_bytes, final_setup_info):
    acc_msg_bytes = gcl_message.MSG_ACC.to_bytes(1, 'big')
    if len(wtp_txid_bytes) != 32:
        raise Exception("wtp_txid must be 32 bytes long")
    if len(sender_key.public_key) != 33:
        raise Exception("public_key_bytes must be 33 bytes long")

    setup_info_bytes = final_setup_info.encode()
    signed_wtp_txid = sender_key.sign(wtp_txid_bytes)

    msg_size = len(setup_info_bytes).to_bytes(1, 'big')
    signed_txid_size = len(signed_wtp_txid).to_bytes(1, 'big')

    message_body = [
        acc_msg_bytes,
        wtp_txid_bytes,
        msg_size,
        signed_txid_size,
        setup_info_bytes,
        signed_wtp_txid
    ]
    message_body_bytes = b''.join(message_body)
    message_body_len = len(message_body_bytes)

    message_parts = [
        _get_prefix_bytes(),
        _get_op_pushdata_bytes(message_body_len),
        _get_version_bytes(),
        message_body_bytes
    ]

    message = b''.join(message_parts)
    return message


def create_reject_wtp_message(sender_key, wtp_txid_bytes, rejection_msg):
    rej_msg_bytes = gcl_message.MSG_REJ.to_bytes(1, 'big')
    if len(wtp_txid_bytes) != 32:
        raise Exception("wtp_txid must be 32 bytes long")
    if len(sender_key.public_key) != 33:
        raise Exception("public_key_bytes must be 33 bytes long")

    rejection_msg_bytes = rejection_msg.encode()
    signed_wtp_txid = sender_key.sign(wtp_txid_bytes)

    msg_size = len(rejection_msg_bytes).to_bytes(1, 'big')
    signed_txid_size = len(signed_wtp_txid).to_bytes(1, 'big')

    message_body = [
        rej_msg_bytes,
        wtp_txid_bytes,
        msg_size,
        signed_txid_size,
        rejection_msg_bytes,
        signed_wtp_txid
    ]
    message_body_bytes = b''.join(message_body)
    message_body_len = len(message_body_bytes)

    message_parts = [
        _get_prefix_bytes(),
        _get_op_pushdata_bytes(message_body_len),
        _get_version_bytes(),
        message_body_bytes
    ]

    message = b''.join(message_parts)
    return message


def create_cancel_lfg_message(sender_key, lfg_txid_bytes, cancel_msg):
    can_msg_bytes = gcl_message.MSG_CAN.to_bytes(1, 'big')
    if len(lfg_txid_bytes) != 32:
        raise Exception("lfg_txid must be 32 bytes long")
    if len(sender_key.public_key) != 33:
        raise Exception("public_key_bytes must be 33 bytes long")

    cancel_msg_bytes = cancel_msg.encode()
    signed_lfg_txid = sender_key.sign(lfg_txid_bytes)

    msg_size = len(cancel_msg_bytes).to_bytes(1, 'big')
    signed_txid_size = len(signed_lfg_txid).to_bytes(1, 'big')

    message_body = [
        can_msg_bytes,
        lfg_txid_bytes,
        msg_size,
        signed_txid_size,
        cancel_msg_bytes,
        signed_lfg_txid
    ]
    message_body_bytes = b''.join(message_body)
    message_body_len = len(message_body_bytes)

    message_parts = [
        _get_prefix_bytes(),
        _get_op_pushdata_bytes(message_body_len),
        _get_version_bytes(),
        message_body_bytes
    ]

    message = b''.join(message_parts)
    return message




from bitcash import format
import gcl_message


def parse_op_push_data(push_data_bytes):
    if push_data_bytes[0] == gcl_message.OP_PUSHDATA:
        size = push_data_bytes[1]
        return push_data_bytes[2:2+size+1]


OP_RETURN_PREFIX_BYTES = 0x6A
def parse_op_return(op_return_bytes, sender_public_key=None):
    # pull out GCL prefix
    if op_return_bytes[0] == OP_RETURN_PREFIX_BYTES and \
       op_return_bytes[1] == 0x04 and \
       op_return_bytes[2] == 0x00 and \
       op_return_bytes[3] == 0x00 and \
       op_return_bytes[4] == 0x13 and \
       op_return_bytes[5] == 0x37:
        op_push_data = parse_op_push_data(op_return_bytes[6:])
        return parse_gcl_message(op_push_data, sender_public_key)

    return None


def parse_lfg(msg_payload):
    public_key_bytes = msg_payload[:33]
    msg_data_size = msg_payload[33]
    signed_msg_data_size = msg_payload[34]
    signed_msg_data_offset = 35 + msg_data_size

    msg_data = msg_payload[35:35 + msg_data_size]
    signed_msg_data = msg_payload[signed_msg_data_offset:signed_msg_data_offset + signed_msg_data_size]

    valid = format.verify_sig(signed_msg_data, msg_data, public_key_bytes)
    if valid:
        msg_data_str = msg_data.decode()
        msg_lfg = gcl_message.MsgLfg(public_key_bytes, msg_data_str)
        return gcl_message.MSG_LFG, msg_lfg

    return None


def parse_wtp(msg_payload):
    lfg_txid_bytes = msg_payload[:32]
    public_key_bytes = msg_payload[32:32+33]
    msg_data_size = msg_payload[32+33]
    signed_txid_size = msg_payload[32+33+1]

    msg_data_offset = 32 + 33 + 1 + 1
    msg_data = msg_payload[msg_data_offset:msg_data_offset + msg_data_size]
    signed_txid = msg_payload[msg_data_offset + msg_data_size:msg_data_offset + msg_data_size + signed_txid_size]

    valid = format.verify_sig(signed_txid, lfg_txid_bytes, public_key_bytes)
    if valid:
        msg_data_str = msg_data.decode()
        msg_wtp = gcl_message.MsgWtp(public_key_bytes, lfg_txid_bytes, msg_data_str)
        return gcl_message.MSG_WTP, msg_wtp

    return None


def parse_acc(msg_payload, sender_public_key_bytes):
    wtp_txid_bytes = msg_payload[:32]
    msg_data_size = msg_payload[32]
    signed_txid_size = msg_payload[33]

    msg_data_offset = 34
    msg_data = msg_payload[msg_data_offset:msg_data_offset + msg_data_size]
    signed_txid = msg_payload[msg_data_offset + msg_data_size:msg_data_offset + msg_data_size + signed_txid_size]

    valid = format.verify_sig(signed_txid, wtp_txid_bytes, sender_public_key_bytes)
    if valid:
        msg_data_str = msg_data.decode()
        msg_acc = gcl_message.MsgAcc(wtp_txid_bytes, msg_data_str)
        return gcl_message.MSG_ACC, msg_acc

    return None


def parse_rej(msg_payload, sender_public_key_bytes):
    wtp_txid_bytes = msg_payload[:32]
    msg_data_size = msg_payload[32]
    signed_txid_size = msg_payload[33]

    msg_data_offset = 34
    msg_data = msg_payload[msg_data_offset:msg_data_offset + msg_data_size]
    signed_txid = msg_payload[msg_data_offset + msg_data_size:msg_data_offset + msg_data_size + signed_txid_size]

    valid = format.verify_sig(signed_txid, wtp_txid_bytes, sender_public_key_bytes)
    if valid:
        msg_data_str = msg_data.decode()
        msg_rej = gcl_message.MsgRej(wtp_txid_bytes, msg_data_str)
        return gcl_message.MSG_REJ, msg_rej

    return None


def parse_can(msg_payload, sender_public_key_bytes):
    txid_bytes = msg_payload[:32]
    msg_data_size = msg_payload[32]
    signed_txid_size = msg_payload[33]

    msg_data_offset = 34
    msg_data = msg_payload[msg_data_offset:msg_data_offset + msg_data_size]
    signed_txid = msg_payload[msg_data_offset + msg_data_size:msg_data_offset + msg_data_size + signed_txid_size]

    valid = format.verify_sig(signed_txid, txid_bytes, sender_public_key_bytes)
    if valid:
        msg_data_str = msg_data.decode()
        msg_can = gcl_message.MsgCan(msg_data_str)
        return gcl_message.MSG_CAN, msg_can

    return None


def parse_gcl_message(gcl_bytes, sender_public_key):
    if gcl_bytes[0] != 0x01:
        return None

    msg_type = gcl_bytes[1]
    msg_payload = gcl_bytes[2:]

    if msg_type == gcl_message.MSG_LFG:
        return parse_lfg(msg_payload)

    if msg_type == gcl_message.MSG_WTP:
        return parse_wtp(msg_payload)

    if msg_type == gcl_message.MSG_ACC:
        return parse_acc(msg_payload, sender_public_key)

    if msg_type == gcl_message.MSG_REJ:
        return parse_rej(msg_payload, sender_public_key)

    if msg_type == gcl_message.MSG_CAN:
        return parse_can(msg_payload, sender_public_key)

    return None


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="gamechain-lobby",
    version="0.0.1",
    author="Albert Boehmler",
    author_email="ajboehmler@gmail.com",
    description="Game matchmaking on the BCH blockchain",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/devalbo/gamechain-lobby",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ),
    install_requires=[]
)


import requests
import time


def get_tx_testnet(txid):
    url = f"https://test-bch-insight.bitpay.com/api/tx/{txid}"
    while True:
        response = requests.get(url)
        if response.status_code == 200:
            return response.json()
        else:
            time.sleep(0.3)


def ensure_prefixed_address(addr):
    if not addr.startswith("bchtest:"):
        return f"bchtest:{addr}"

    return addr


def get_transaction_ids_for_address(addr):
    # https://test-bch-insight.bitpay.com/api/addr/bchtest:qqngkfyr38e6gp8cmmzejfdc47h3ppum5qflqpr87y
    # {
    #     "addrStr": "qqngkfyr38e6gp8cmmzejfdc47h3ppum5qflqpr87y",
    #     "balance": 0,
    #     "balanceSat": 0,
    #     "totalReceived": 0,
    #     "totalReceivedSat": 0,
    #     "totalSent": 0,
    #     "totalSentSat": 0,
    #     "unconfirmedBalance": 0,
    #     "unconfirmedBalanceSat": 0,
    #     "unconfirmedTxApperances": 0,
    #     "txApperances": 1,
    #     "transactions": [
    #         "bd4282b683102bf0e2e4fb025d27c88511a3c7aab4c911d73e00ce9553e1ed01"
    #     ]
    # }
    testnet_addr_url = "https://test-bch-insight.bitpay.com/api/addr/" + ensure_prefixed_address(addr)
    addr_data = requests.get(testnet_addr_url).json()
    txids = addr_data["transactions"]
    return txids


def single(items, predicate, raiseExceptionOnNotFound=True):
    filtered = [i for i in items if predicate(i)]
    if len(filtered) == 0:
        if raiseExceptionOnNotFound:
            raise Exception("No item that matches predicate %s" % predicate)
        return None
    if len(filtered) > 1:
        raise Exception("More than one item matches predicate %s" % predicate)

    return filtered[0]


def where(items, predicate):
    filtered = [i for i in items if predicate(i)]
    return filtered


if __name__ == "__main__":
    tx = get_tx_testnet("91955d1d228453fac7113850eeab48bf31350f5e0767a7d2357ca47af1836cde")

    print(tx)



import threading
from socketIO_client import SocketIO, LoggingNamespace
import cashaddress
import game_shelves

SOCKETIO_SERVER = "https://test-bch-insight.bitpay.com"
# SOCKETIO_SERVER = "https://bch-insight.bitpay.com"


class GameChainMonitor:

    def __init__(self, wait_time=60):
        self._tx_handler = None
        monitor_thread = threading.Thread(target=self._start_monitor)
        monitor_thread.start()

    def _start_monitor(self):
        self._socketIO = SocketIO(SOCKETIO_SERVER, namespace=LoggingNamespace)
        self._socketIO.on('connect', self.on_connect)
        self._socketIO.on('disconnect', self.on_disconnect)
        self._socketIO.on('reconnect', self.on_reconnect)
        self._socketIO.on("tx", self.on_tx)

        self._socketIO.wait()

    def on_connect(self):
        print('connect')
        self._socketIO.emit('subscribe', "inv");

    def on_disconnect(self):
        print('disconnect')

    def on_reconnect(self):
        print('reconnect')

    def on_aaa_response(*args):
        print('on_aaa_response', args)

    def on_tx(self, data):
        if self._tx_handler is not None:
            self._tx_handler(data)

    def register_tx_handler(self, tx_handler):
        self._tx_handler = tx_handler

    def un_register_tx_handler(self, tx_handler):
        if self._tx_handler is tx_handler:
            self._tx_handler = None
        else:
            raise Exception("Invalid tx handler")

    def stop(self):
        if hasattr(self, "_socketIO"):
            self._socketIO.disconnect()




    # socketIO = SocketIO(SOCKETIO_SERVER, namespace=LoggingNamespace)
    # socketIO.on('connect', on_connect)
    # socketIO.on('disconnect', on_disconnect)
    # socketIO.on('reconnect', on_reconnect)
    #
    # socketIO.on("tx", on_tx)

    # # Listen
    # socketIO.on('aaa_response', on_aaa_response)
    # socketIO.emit('aaa')
    # socketIO.emit('aaa')
    # socketIO.wait(seconds=1)
    #
    # # Stop listening
    # socketIO.off('aaa_response')
    # socketIO.emit('aaa')
    # socketIO.wait(seconds=1)
    #
    # # Listen only once
    # socketIO.once('aaa_response', on_aaa_response)
    # socketIO.emit('aaa')  # Activate aaa_response
    # socketIO.emit('aaa')  # Ignore
    # socketIO.wait(seconds=100)



from socketIO_client import SocketIO, LoggingNamespace

SOCKETIO_SERVER = "https://test-bch-insight.bitpay.com"
SOCKETIO_SERVER = "https://bch-insight.bitpay.com"

def on_connect():
    print('connect')
    socketIO.emit('subscribe', "inv");

def on_disconnect():
    print('disconnect')

def on_reconnect():
    print('reconnect')

def on_aaa_response(*args):
    print('on_aaa_response', args)

def on_tx(data):
    print(f"TX: {data}")


socketIO = SocketIO(SOCKETIO_SERVER, namespace=LoggingNamespace)
socketIO.on('connect', on_connect)
socketIO.on('disconnect', on_disconnect)
socketIO.on('reconnect', on_reconnect)

socketIO.on("tx", on_tx)

# # Listen
# socketIO.on('aaa_response', on_aaa_response)
# socketIO.emit('aaa')
# socketIO.emit('aaa')
# socketIO.wait(seconds=1)
#
# # Stop listening
# socketIO.off('aaa_response')
# socketIO.emit('aaa')
# socketIO.wait(seconds=1)
#
# # Listen only once
# socketIO.once('aaa_response', on_aaa_response)
# socketIO.emit('aaa')  # Activate aaa_response
# socketIO.emit('aaa')  # Ignore
socketIO.wait(seconds=100)